<?xml version="1.0" encoding="UTF-8"?>
<!-- xsl script to convert lsmd v0.3p files to lsdoc format (version unknown) -->
<!-- Created 2003 by Joshua Seagoe.  This file is in the Public Domain. -->
<!-- Notes:
  The generated files probably will not validate to the DTD that I've been
  given, but I haven't seen any lsdoc files that do either.  I'm assuming
  that this DTD wasn't completely finished.

  This will handle only relatively simple data-paths:
    - each data element must have a unique id, not just a unique path
    - the @data attribute on the parameter must not conain any slashes
  Otherwise nested types should work ok, you just have to select the rest of
  the path on the corresponding group or command elements.  In general this
  is a good style to use anyway, since it makes it a lot easier for other
  xsl-based tools to read the file.

  This does not convert sections to description/@header attributes.  It's
  probably possible, but I'm not sure how this is supposed to work in lsdoc.

  This does not use group elements in the generated file.  Again, I'm not
  sure how these are supposed to work in lsdoc.
  
  The changelog element in the generated file corresponds to the history
  of the module.  The lsmd/history element (document history) is completely
  ignored.  (This could be fixed without too much difficulty.)
  
  This doesn't translate the copyright element, because I'm not sure if
  module/copyright in lsdoc corresponds to lsmd/copyright in lsmd or to
  module/copyright.  The latter would make sense, but the examples that I
  have seem to be equivalent to lsmd/copyright.
  -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<!-- lsmd -> lsdoc -->
  <xsl:template match="lsmd">
    <xsl:choose>
      <xsl:when test="@spec != '0.3p'">
        <xsl:comment> Translation may not be accurate: @spec was "<xsl:value-of select="@spec"/>", expected "0.3p" </xsl:comment>
        <xsl:message> Translation may not be accurate: @spec was "<xsl:value-of select="@spec"/>", expected "0.3p" </xsl:message>
      </xsl:when>
      <xsl:when test="not(@spec)">
        <xsl:comment> Translation may not be accurate: @spec was missing, expected "0.3p" </xsl:comment>
        <xsl:message> Translation may not be accurate: @spec was missing, expected "0.3p" </xsl:message>
      </xsl:when>
    </xsl:choose>
    <xsl:element name="lsdoc">
      <xsl:comment> Converted from lsmd file with lsmd2lsdoc.xsl </xsl:comment>
      <xsl:apply-templates select="module" mode="convert"/>
    </xsl:element>
  </xsl:template>
  
<!-- module -> module -->
  <xsl:template match="module" mode="convert">
    <xsl:element name="module">
      <xsl:if test="@friendlyname">
        <xsl:attribute name="friendlyname"><xsl:value-of select="@friendlyname"/></xsl:attribute>
      </xsl:if>
      <xsl:if test="@version">
        <xsl:attribute name="version"><xsl:value-of select="@version"/></xsl:attribute>
      </xsl:if>
      <xsl:if test="@category">
        <xsl:attribute name="type"><xsl:value-of select="@category"/></xsl:attribute>
      </xsl:if>
      <!-- build -> @filename, @size, @crc -->
      <xsl:apply-templates select="build/file" mode="filename"/>
      <!-- @load-free, @load-wharf -> @load -->
      <xsl:if test="@load-free or @load-wharf">
        <xsl:choose>
          <xsl:when test="(@load-free='true' or not(@load-free)) and @load-wharf='true'">
            <xsl:attribute name="load">mixed-mode</xsl:attribute>
          </xsl:when>
          <xsl:when test="@load-free='false' and @load-wharf='true'">
            <xsl:attribute name="load">wharf</xsl:attribute>
          </xsl:when>
          <xsl:otherwise>
            <xsl:attribute name="load">loadmodule</xsl:attribute>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>

      <xsl:apply-templates select="author" mode="convert"/>
      <xsl:apply-templates select="documentation/description" mode="convert"/>
      <xsl:apply-templates select="config|control|variables" mode="convert"/>
      <xsl:apply-templates select="history" mode="convert"/>
    </xsl:element>
  </xsl:template>
  
<!-- file -> @filename, @size, @crc -->
  <xsl:template match="file" mode="filename">
    <xsl:if test="position()=1">
      <xsl:attribute name="filename"><xsl:value-of select="@name"/></xsl:attribute>
      <xsl:if test="@size">
        <xsl:attribute name="size"><xsl:value-of select="@size"/></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="hash" mode="filename"/>
    </xsl:if>
  </xsl:template>
<!-- hash -> @crc -->
  <xsl:template match="hash" mode="filename">
    <xsl:if test="@method='crc32'">
      <xsl:attribute name="crc"><xsl:value-of select="text()"/></xsl:attribute>
    </xsl:if>
  </xsl:template>

<!-- description -> description -->
  <xsl:template match="description" mode="convert">
    <xsl:element name="description">
      <xsl:apply-templates select="*|text()"/>
    </xsl:element>
  </xsl:template>
  <!-- p -> br (loss of structure) -->
  <xsl:template match="p">
    <xsl:apply-templates select="*|text()"/>
    <xsl:if test="position()!=last()">
      <xsl:element name="br"/>
    </xsl:if>
  </xsl:template>
  <!-- em -> i -->
  <xsl:template match="em">
    <xsl:element name="i">
      <xsl:apply-templates select="*|text()"/>
    </xsl:element>
  </xsl:template>
  <!-- strong -> b -->
  <xsl:template match="strong">
    <xsl:element name="b">
      <xsl:apply-templates select="*|text()"/>
    </xsl:element>
  </xsl:template>
  <!-- link -> url -->
  <xsl:template match="link">
    <xsl:element name="url">
      <xsl:attribute name="src"><xsl:value-of select="@href"/></xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </xsl:element>
  </xsl:template>
  <!-- xref, code -> no equivalent -->
  <!-- no equivalent -> u -->

<!-- author -> author -->
  <xsl:template match="author" mode="convert">
    <xsl:element name="author">
      <xsl:choose>
        <xsl:when test="@name">
          <xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="name"><xsl:value-of select="text()"/></xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="@email">
        <xsl:attribute name="email"><xsl:value-of select="@email"/></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="contact|link" mode="author"/>
    </xsl:element>
  </xsl:template>
  <!-- contact -> @icq, @msn, @yim, @aim -->
  <xsl:template match="contact" mode="author">
    <xsl:choose>
      <xsl:when test="@service='icq'">
        <xsl:attribute name="icq"><xsl:value-of select="text()"/></xsl:attribute>
      </xsl:when>
      <xsl:when test="@service='msn'">
        <xsl:attribute name="msn"><xsl:value-of select="text()"/></xsl:attribute>
      </xsl:when>
      <xsl:when test="@service='yim'">
        <xsl:attribute name="yim"><xsl:value-of select="text()"/></xsl:attribute>
      </xsl:when>
      <xsl:when test="@service='aim'">
        <xsl:attribute name="aim"><xsl:value-of select="text()"/></xsl:attribute>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  <!-- link -> @website -->
  <xsl:template match="link" mode="author">
    <xsl:choose>
      <xsl:when test="@rel='home'">
        <xsl:attribute name="website"><xsl:value-of select="@href"/></xsl:attribute>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
<!-- history -> changelog -->
  <xsl:template match="history" mode="convert">
    <xsl:element name="changelog">
      <xsl:apply-templates select="revision" mode="convert"/>
    </xsl:element>
  </xsl:template>
<!-- revision -> version -->
  <xsl:template match="revision" mode="convert">
    <xsl:element name="version">
      <xsl:attribute name="id"><xsl:value-of select="@version"/></xsl:attribute>
      <xsl:apply-templates select="note|changed|added|removed|fixed" mode="convert"/>
    </xsl:element>
  </xsl:template>
<!-- note -> description -->
  <xsl:template match="note" mode="convert">
    <xsl:element name="description">
      <xsl:apply-templates select="*|text()"/>
    </xsl:element>
  </xsl:template>
<!-- changed, added, removed, fixed -> entry -->
  <xsl:template match="changed|added|removed|fixed" mode="convert">
    <xsl:element name="entry">
      <xsl:apply-templates select="*|text()"/>
    </xsl:element>
  </xsl:template>
  
<!-- config -> flat -->
  <xsl:template match="config" mode="convert">
    <xsl:apply-templates select=".//single|multiple" mode="convert"/>
  </xsl:template>
<!-- control -> flat -->
  <xsl:template match="control" mode="convert">
    <xsl:apply-templates select=".//bang" mode="convert"/>
  </xsl:template>
<!-- variables -> flat -->
  <xsl:template match="variables" mode="convert">
    <xsl:apply-templates select=".//var" mode="convert"/>
  </xsl:template>

<!-- key to resolve prefixes -->
  <xsl:key name="prefix" match="key" use="@id"/>

<!-- single -> command -->
  <xsl:template match="single" mode="convert">
    <xsl:element name="command">
      <xsl:attribute name="name">
        <!-- resolve prefixes -->
        <xsl:for-each select="ancestor-or-self::*/@prefix">
          <xsl:choose>
            <xsl:when test="key('prefix',.)/@default">
              <xsl:value-of select="key('prefix',.)/@default"/>
            </xsl:when>
            <xsl:otherwise>
              <span class="prefix">
                <xsl:value-of select="key('prefix',.)/@id"/>
              </span>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        <!-- the actual name -->
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:apply-templates select="*" mode="convert"/>
    </xsl:element>
  </xsl:template>

<!-- multiple -> definition -->
  <xsl:template match="multiple" mode="convert">
    <xsl:element name="definition">
      <xsl:attribute name="name">
        <!-- does it start with a *? -->
        <xsl:choose>
          <xsl:when test="@star='false'"></xsl:when>
          <xsl:otherwise><xsl:text>*</xsl:text></xsl:otherwise>
        </xsl:choose>
        <!-- resolve prefixes -->
        <xsl:for-each select="ancestor-or-self::*/@prefix">
          <xsl:choose>
            <xsl:when test="key('prefix',.)/@default">
              <xsl:value-of select="key('prefix',.)/@default"/>
            </xsl:when>
            <xsl:otherwise>
              <span class="prefix">
                <xsl:value-of select="key('prefix',.)/@id"/>
              </span>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        <!-- the actual name -->
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:apply-templates select="*" mode="convert"/>
    </xsl:element>
  </xsl:template>
 
<!-- bang -> bang -->
  <xsl:template match="bang" mode="convert">
    <xsl:element name="bang">
      <xsl:attribute name="name">
        <!-- resolve prefixes -->
        <xsl:for-each select="ancestor-or-self::*/@prefix">
          <xsl:choose>
            <xsl:when test="key('prefix',.)/@default">
              <xsl:value-of select="key('prefix',.)/@default"/>
            </xsl:when>
            <xsl:otherwise>
              <span class="prefix">
                <xsl:value-of select="key('prefix',.)/@id"/>
              </span>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        <!-- the actual name -->
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:apply-templates select="*" mode="convert"/>
    </xsl:element>
  </xsl:template>
 
<!-- var -> evar -->
  <xsl:template match="var" mode="convert">
    <!-- only convert $evars$, I don't know if evar is appropriate for others -->
    <xsl:if test="not(ancestor-or-self::*/@delim-begin) or (ancestor-or-self::*/@delim-begin='$')">
      <xsl:if test="not(ancestor-or-self::*/@delim-end) or (ancestor-or-self::*/@delim-end='$')">
        <xsl:element name="evar">
          <xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
          <xsl:apply-templates select="*" mode="convert"/>
        </xsl:element>
      </xsl:if>
    </xsl:if>
  </xsl:template>


<!-- data types -->

  <!-- key to resolve data references -->
  <xsl:key name="data" match="*" use="@id"/>

  <!-- action -> action -->
  <xsl:template match="action" mode="convert">
    <xsl:element name="action">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- bool -> bool -->
  <xsl:template match="bool" mode="convert">
    <xsl:element name="bool">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- color -> color -->
  <xsl:template match="color" mode="convert">
    <xsl:element name="color">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- coordinate -> integer -->
  <xsl:template match="coordinate" mode="convert">
    <xsl:element name="integer">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
      <xsl:call-template name="get-minmax"/>
      <xsl:attribute name="accept-negative">true</xsl:attribute>
      <xsl:call-template name="get-origin"/>
    </xsl:element>
  </xsl:template>
  <!-- file -> file -->
  <xsl:template match="file" mode="convert">
    <xsl:element name="file">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
      <xsl:if test="@data">
        <xsl:if test="key('data',@data)/item">
          <xsl:attribute name="mask">
            <xsl:for-each select="key('data',@data)/item">
              <xsl:value-of select="@value"/>
              <xsl:if test="position()!=last()">
                <!-- I assume this is the separator for @mask -->
                <xsl:text>|</xsl:text>
              </xsl:if>
            </xsl:for-each>
          </xsl:attribute>
        </xsl:if>
      </xsl:if>
    </xsl:element>
  </xsl:template>
  <!-- folder -> folder -->
  <xsl:template match="folder" mode="convert">
    <xsl:element name="folder">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- font -> font -->
  <xsl:template match="font" mode="convert">
    <xsl:element name="font">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- image -> image -->
  <xsl:template match="image" mode="convert">
    <xsl:element name="image">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- integer -> integer -->
  <xsl:template match="integer" mode="convert">
    <xsl:element name="integer">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
      <xsl:call-template name="get-minmax"/>
      <xsl:call-template name="get-signed"/>
    </xsl:element>
  </xsl:template>
  <!-- option -> list -->
  <xsl:template match="option" mode="convert">
    <xsl:element name="list">
      <xsl:call-template name="get-optional"/>
      <xsl:if test="@data">
        <xsl:variable name="default"><xsl:value-of select="key('data',@data)/@default"/></xsl:variable>
        <xsl:if test="key('data',@data)/item">
          <xsl:for-each select="key('data',@data)/item">
            <xsl:element name="item">
              <xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
              <xsl:choose>
                <xsl:when test="@value">
                  <xsl:attribute name="value"><xsl:value-of select="@value"/></xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="value"><xsl:value-of select="@name"/></xsl:attribute>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:if test="@name = $default">
                <xsl:attribute name="default">true</xsl:attribute>
              </xsl:if>
            </xsl:element>
          </xsl:for-each>
        </xsl:if>
      </xsl:if>
    </xsl:element>
  </xsl:template>
  <!-- sound -> file -->
  <xsl:template match="sound" mode="convert">
    <xsl:element name="file">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- string -> string -->
  <xsl:template match="string" mode="convert">
    <xsl:element name="string">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>
  <!-- url -> string -->
  <xsl:template match="url" mode="convert">
    <xsl:element name="string">
      <xsl:call-template name="get-optional"/>
      <xsl:call-template name="get-default"/>
    </xsl:element>
  </xsl:template>

  <!-- copy @optional attribute -->
  <xsl:template name="get-optional">
    <xsl:if test="@optional">
      <xsl:attribute name="optional"><xsl:value-of select="@optional"/></xsl:attribute>
    </xsl:if>
  </xsl:template>
  <!-- copy @default from the linked data element -->
  <xsl:template name="get-default">
    <!-- if it doesn't have data, we can skip a lot -->
    <xsl:if test="@data">
      <xsl:if test="key('data',@data)/@default">
        <xsl:attribute name="default"><xsl:value-of select="key('data',@data)/@default"/></xsl:attribute>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <!-- @min,@max -> @minvalue,@maxvalue -->
  <xsl:template name="get-minmax">
    <xsl:if test="@data">
      <xsl:if test="key('data',@data)/@min">
        <xsl:attribute name="minvalue"><xsl:value-of select="key('data',@data)/@min"/></xsl:attribute>
      </xsl:if>
      <xsl:if test="key('data',@data)/@max">
        <xsl:attribute name="maxvalue"><xsl:value-of select="key('data',@data)/@max"/></xsl:attribute>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <!-- @origin -> type -->
  <xsl:template name="get-origin">
    <xsl:if test="@data">
      <xsl:choose>
        <xsl:when test="key('data',@data)/@origin">
          <xsl:attribute name="type">relative</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="type">absolute</xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>
  <!-- @signed -> @accept-negative -->
  <xsl:template name="get-signed">
    <xsl:if test="@data">
      <xsl:if test="key('data',@data)/@signed">
        <xsl:attribute name="accept-negative"><xsl:value-of select="key('data',@data)/@signed"/></xsl:attribute>
      </xsl:if>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
